#include <linux/mm.h>
#include <linux/malloc.h>
#include <linux/tcfs.h>

const char uu_base64[64] =
{
  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
  'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
  'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
  'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
  'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
  'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
  'w', 'x', 'y', 'z', '0', '1', '2', '3',
  '4', '5', '6', '7', '8', '9', '+', '#'
};

#define ENC(Char) (uu_base64[(Char) & 077])
#define	DEC(Char) (((Char) - ' ') & 077)

void tcfsencode (char *buffer, int len)
{
  int ch, n,i=0,j,end=2+4*(len/3+((len%3)?1:0));
  char *p;
  char bufret[end];

  n = len;

  for (p=buffer,j=0;j<n;j+=3,p+=3)
  {
	ch = *p >> 2;
	ch = ENC (ch);
	bufret[i++]=ch;
	ch = ((*p << 4) & 060) | ((p[1] >> 4) & 017);
	ch = ENC (ch);
	bufret[i++]=ch;
	ch = ((p[1] << 2) & 074) | ((p[2] >> 6) & 03);
	ch = ENC (ch);
	bufret[i++]=ch;
	ch = p[2] & 077;
	ch = ENC (ch);
	bufret[i++]=ch;
  }

  bufret[i]='=';
  bufret[i+1]='\0';
  memcpy(buffer, bufret, i+2);
}

static const char b64_tab[256] =
{
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*000-007*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*010-017*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*020-027*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*030-037*/
    '\177', '\177', '\177', '\77',  '\177', '\177', '\177', '\177', /*040-047*/
    '\177', '\177', '\177', '\76',  '\177', '\177', '\177', '\177', /*050-057*/
    '\64',  '\65',  '\66',  '\67',  '\70',  '\71',  '\72',  '\73',  /*060-067*/
    '\74',  '\75',  '\177', '\177', '\177', '\100', '\177', '\177', /*070-077*/
    '\177', '\0',   '\1',   '\2',   '\3',   '\4',   '\5',   '\6',   /*100-107*/
    '\7',   '\10',  '\11',  '\12',  '\13',  '\14',  '\15',  '\16',  /*110-117*/
    '\17',  '\20',  '\21',  '\22',  '\23',  '\24',  '\25',  '\26',  /*120-127*/
    '\27',  '\30',  '\31',  '\177', '\177', '\177', '\177', '\177', /*130-137*/
    '\177', '\32',  '\33',  '\34',  '\35',  '\36',  '\37',  '\40',  /*140-147*/
    '\41',  '\42',  '\43',  '\44',  '\45',  '\46',  '\47',  '\50',  /*150-157*/
    '\51',  '\52',  '\53',  '\54',  '\55',  '\56',  '\57',  '\60',  /*160-167*/
    '\61',  '\62',  '\63',  '\177', '\177', '\177', '\177', '\177', /*170-177*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*200-207*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*210-217*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*220-227*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*230-237*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*240-247*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*250-257*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*260-267*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*270-277*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*300-307*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*310-317*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*320-327*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*330-337*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*340-347*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*350-357*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*360-367*/
    '\177', '\177', '\177', '\177', '\177', '\177', '\177', '\177', /*370-377*/
};

int tcfsdecode (char *buf, int len)
{
  int i=0;
  char bufret[len];
  char *p=buf;

  /* The following implementation of the base64 decoding might look
	a bit clumsy but I only try to follow the POSIX standard:
	 ``All line breaks or other characters not found in the table
	   [with base64 characters] shall be ignored by decoding
	   software.''  */

  while (*p!='=')
  {
    char c1, c2, c3;

    while ((b64_tab[(int)*p] & '\100') != 0)
	p++;

    c1=b64_tab[(int)*p++];

    while ((b64_tab[(int)*p] & '\100') !=0 )
	p++;

    c2=b64_tab[(int)*p++];

    while (b64_tab[(int)*p] == '\177')
	p++;

    if (*p == '=')
      {
	bufret[i++]= (c1<<2 | c2>>4);
        break;
      }

    c3=b64_tab[(int)*p++];

    while (b64_tab[(int)*p] == '\177')
	p++;

    bufret[i++]= (c1<<2 | c2>>4);
    bufret[i++]= (c2<<4 | c3>>2);

    if (*p == '=')
	break;
    else
	bufret[i++]= (c3<<6|b64_tab[(int)*p++]);
  }

  memcpy (buf, bufret, i+1);
  return i+1;
}

/*int main(void)
{
	int i,j,n;
	char in[1024],out[1024];
	for (i=1;i<1023;i++){
		for (j=0;j<i;j++) {
			in[j]='a';
		}
		in[i]='\0';
		strcpy(out,in);
		tcfsencode(in,i+1);
		tcfsdecode(in,strlen(in)+1);
		if (strcmp(in,out)!=0) {
			printf("\nerrore %d %s vs %s\n",i,in,out);
		} else {
			printf("%d ",i);
			fflush(stdout);
		}	
	}
	return 0;
}*/